pub struct Session<RW> { /* private fields */ }Expand description
The object that handles the communication between two endpoints using the Honk-RPC protocol. Provides methods for setting and getting configuration parameters, reading and processing message documents, and handling API requests and responses.
Implementations
Sourceimpl<RW> Session<RW>
impl<RW> Session<RW>
Sourcepub fn set_max_message_size(
&mut self,
max_message_size: i32,
) -> Result<(), Error>
pub fn set_max_message_size( &mut self, max_message_size: i32, ) -> Result<(), Error>
Sets the maximum message size this Session is willing to read from from the underlying RW. Attempted reads will abort if the next bson document’s i32 size field is greater than the max_message_size defined in this function.
Sourcepub fn get_max_message_size(&self) -> usize
pub fn get_max_message_size(&self) -> usize
Gets the maximum allowed message size this Session is willing to read from the underlying RW. The default value is 4096 bytes.
Sourcepub fn set_max_wait_time(&mut self, max_wait_time: Duration)
pub fn set_max_wait_time(&mut self, max_wait_time: Duration)
Sets the maximum amount of time this Session is willing to wait for a new Honk-RPC message on the underlying RW. Session updates will fil after max_wait_time has elapsed without receiving any new Honk-RPC message documents.
Sourcepub fn get_max_wait_time(&self) -> Duration
pub fn get_max_wait_time(&self) -> Duration
Gets the maximum amount this Session is willing to wait for a new Honk-RPC message. The default value is 60 seconds.
Sourcepub fn into_stream(self) -> RW
pub fn into_stream(self) -> RW
Consumes the Session and returns the underlying stream.
Sourcepub fn update(
&mut self,
apisets: Option<&mut [&mut dyn ApiSet]>,
) -> Result<(), Error>
pub fn update( &mut self, apisets: Option<&mut [&mut dyn ApiSet]>, ) -> Result<(), Error>
Read and process Honk-RPC message documents from connected peer, handle any new incoming Honk-RPC requests, update any in-progress async requests and write pending reponses, errors and requests to peer. This function must be called regularly for the Session to make forward progress.
Sourcepub fn client_call(
&mut self,
namespace: &str,
function: &str,
version: i32,
arguments: Document,
) -> Result<RequestCookie, Error>
pub fn client_call( &mut self, namespace: &str, function: &str, version: i32, arguments: Document, ) -> Result<RequestCookie, Error>
Performs a client call to a remote function. Returns a RequestCookie to associate this client call with a future Response.
Sourcepub fn client_drain_responses(&mut self) -> Drain<'_, Response>
pub fn client_drain_responses(&mut self) -> Drain<'_, Response>
Drains all Response objects resulting from prevoius invocations of Session::client_call()
Sourcepub fn client_next_response(&mut self) -> Option<Response>
pub fn client_next_response(&mut self) -> Option<Response>
Retrieves the next Response object from previous invocations of Session::client_call()